@@ -22,11 +22,11 @@ IO::Iron::IronMQ::Api - IronMQ API reference for Perl Client Libraries!
 
 =head1 VERSION
 
-Version 0.01_04
+Version 0.07
 
 =cut
 
-our $VERSION = '0.01_04';
+our $VERSION = '0.07';
 
 
 =head1 SYNOPSIS
@@ -44,6 +44,8 @@ This package is for internal use of IO::Iron::IronMQ::Client/Queue packages.
 
 =head3 IRONMQ_LIST_MESSAGE_QUEUES
 
+/projects/{Project ID}/queues
+
 =cut
 
 sub IRONMQ_LIST_MESSAGE_QUEUES {
@@ -63,6 +65,8 @@ sub IRONMQ_LIST_MESSAGE_QUEUES {
 
 =head3 IRONMQ_GET_INFO_ABOUT_A_MESSAGE_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}
+
 =cut
 
 sub IRONMQ_GET_INFO_ABOUT_A_MESSAGE_QUEUE {
@@ -80,6 +84,8 @@ sub IRONMQ_GET_INFO_ABOUT_A_MESSAGE_QUEUE {
 
 =head3 IRONMQ_UPDATE_A_MESSAGE_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}
+
 =cut
 
 sub IRONMQ_UPDATE_A_MESSAGE_QUEUE {
@@ -98,6 +104,8 @@ sub IRONMQ_UPDATE_A_MESSAGE_QUEUE {
 
 =head3 IRONMQ_DELETE_A_MESSAGE_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}
+
 =cut
 
 sub IRONMQ_DELETE_A_MESSAGE_QUEUE {
@@ -115,6 +123,8 @@ sub IRONMQ_DELETE_A_MESSAGE_QUEUE {
 
 =head3 IRONMQ_CLEAR_ALL_MESSAGES_FROM_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/clear
+
 =cut
 
 sub IRONMQ_CLEAR_ALL_MESSAGES_FROM_A_QUEUE {
@@ -131,10 +141,32 @@ sub IRONMQ_CLEAR_ALL_MESSAGES_FROM_A_QUEUE {
 		};
 }
 
+=head3 IRONMQ_SET_ALERTS_TO_A_QUEUE
+
+/projects/{Project ID}/queues/{Queue Name}/alerts/
+
+=cut
+
+sub IRONMQ_SET_ALERTS_TO_A_QUEUE {
+	return {
+			'action_name'  => 'IRONMQ_SET_ALERTS_TO_A_QUEUE',
+			'href'         => '{Protocol}://{Host}:{Port}{Host Path Prefix}/projects/{Project ID}/queues/{Queue Name}/alerts',
+			'action'       => 'POST',
+			'return'       => 'MESSAGE',
+			'retry'        => 1,
+			'require_body' => 1,
+			'request_fields' => { 'alerts' => 1 },
+			'url_escape'   => { '{Project ID}' => 1, '{Queue Name}' => 1 },
+			'log_message'  => '(project={Project ID}, queue={Queue Name}). Set alerts to the queue.',
+		};
+}
+
 =head2 Operate messages
 
 =head3 IRONMQ_ADD_MESSAGES_TO_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/messages
+
 =cut
 
 sub IRONMQ_ADD_MESSAGES_TO_A_QUEUE {
@@ -153,6 +185,8 @@ sub IRONMQ_ADD_MESSAGES_TO_A_QUEUE {
 
 =head3 IRONMQ_GET_MESSAGES_FROM_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/messages
+
 =cut
 
 sub IRONMQ_GET_MESSAGES_FROM_A_QUEUE {
@@ -171,6 +205,8 @@ sub IRONMQ_GET_MESSAGES_FROM_A_QUEUE {
 
 =head3 IRONMQ_PEEK_MESSAGES_ON_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/messages/peek
+
 =cut
 
 sub IRONMQ_PEEK_MESSAGES_ON_A_QUEUE {
@@ -189,6 +225,8 @@ sub IRONMQ_PEEK_MESSAGES_ON_A_QUEUE {
 
 =head3 IRONMQ_DELETE_A_MESSAGE_FROM_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/messages/{Message ID}
+
 =cut
 
 sub IRONMQ_DELETE_A_MESSAGE_FROM_A_QUEUE {
@@ -206,6 +244,8 @@ sub IRONMQ_DELETE_A_MESSAGE_FROM_A_QUEUE {
 
 =head3 IRONMQ_DELETE_MULTIPLE_MESSAGES_FROM_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/messages
+
 =cut
 
 sub IRONMQ_DELETE_MULTIPLE_MESSAGES_FROM_A_QUEUE {
@@ -224,6 +264,8 @@ sub IRONMQ_DELETE_MULTIPLE_MESSAGES_FROM_A_QUEUE {
 
 =head3 IRONMQ_TOUCH_A_MESSAGE_ON_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/messages/{Message ID}/touch
+
 =cut
 
 sub IRONMQ_TOUCH_A_MESSAGE_ON_A_QUEUE {
@@ -242,6 +284,8 @@ sub IRONMQ_TOUCH_A_MESSAGE_ON_A_QUEUE {
 
 =head3 IRONMQ_RELEASE_A_MESSAGE_ON_A_QUEUE
 
+/projects/{Project ID}/queues/{Queue Name}/messages/{Message ID}/release
+
 =cut
 
 sub IRONMQ_RELEASE_A_MESSAGE_ON_A_QUEUE {
@@ -267,8 +311,8 @@ Mikko Koivunalho, C<< <mikko.koivunalho at iki.fi> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-net-ironmq at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Net-IronMQ>.  I will be notified, and then you'll
+Please report any bugs or feature requests to C<bug-io-iron at rt.cpan.org>, or through
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=IO-Iron>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 
@@ -285,24 +329,24 @@ You can also look for information at:
 
 =item * RT: CPAN's request tracker (report bugs here)
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Net-IronMQ>
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=IO-Iron>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
-L<http://annocpan.org/dist/Net-IronMQ>
+L<http://annocpan.org/dist/IO-Iron>
 
 =item * CPAN Ratings
 
-L<http://cpanratings.perl.org/d/Net-IronMQ>
+L<http://cpanratings.perl.org/d/IO-Iron>
 
 =item * Search CPAN
 
-L<http://search.cpan.org/dist/Net-IronMQ/>
+L<http://search.cpan.org/dist/IO-Iron/>
 
 =back
 
 
-=head1 ACKNOWLEDGEMENTS
+=head1 ACKNOWLEDGMENTS
 
 Cool idea, "message queue in the cloud": http://www.iron.io/.
 
